#define MAKEWHATIS_C TRUE
/* ========================================================================
 * Name - makewhatis.c
 *
 * Description - hack at a makewhatis, for systems that don't have one
 *
 * $Id: makewhatis.c,v 1.2 1992/09/15 22:37:28 penneyj Exp $
 *
 * $Log: makewhatis.c,v $
 * Revision 1.2  1992/09/15  22:37:28  penneyj
 * misc facelift, preparing for distribution on alt.sources
 *
 * Revision 1.1  1992/09/15  22:18:47  penneyj
 * Initial revision
 *
 *========================================================================
 */

#include "general.h"
#include "config.h"

#define STRINGSIZE 2048
#define FILENAMESIZE 1024 /* MAXPATHLEN, if you want to get picky.... */

#define SKIP_WHITE(p) { \
  while (*(p) == ' ' || *(p) == '\t') \
    (p)++; \
  }

#define SKIP_TEXT(p) { \
  while (*(p) != ' ' && *(p) != '\t' && *(p) != '\0') \
    (p)++; \
  }

#if defined(FLG_PROTOTYPE)
static int acceptLine(FILE *f, char aLine[])
#else
static int acceptLine(f, aLine)
FILE *f;
char aLine[];
#endif
{
if (strncmp(aLine, ".TH", 3) == 0)
  return 1;
if (strncmp(aLine, ".SH", 3) == 0) {
  char *p = &aLine[3];

  /* see if this is a NAME section */
  SKIP_WHITE(p);
  if (strncmp(p, "NAME", 4) != 0)
    return 0;

  /* hack; turn into following line */
  fgets(aLine, STRINGSIZE, f);
  return 2;
  }
return 0;
}

#if defined(FLG_PROTOTYPE)
static void removeBackSpaces(char aLine[])
#else
static void removeBackSpaces(aLine)
char aLine[];
#endif
{
int lineLen = strlen(aLine);
int i;

for (i = 0; i < lineLen; i ++) {
  if (aLine[i] != '\b')
    continue;
  memmove(&aLine[i], &aLine[i + 2], /* skip overstrike as well */
	lineLen - i - 2 + 1 /* include trailing NUL */);
  lineLen = strlen(aLine);
  }
}

#if defined(FLG_PROTOTYPE)
static void removeTextFromLine(char aLine[], const char theText[])
#else
static void removeTextFromLine(aLine, theText)
char aLine[];
const char theText[];
#endif
{
int textLen = strlen(theText);
int lineLen = strlen(aLine);
int i;

for (i = 0; i < lineLen - textLen; i ++) {
  if (strncmp(&aLine[i], theText, textLen) != 0)
    continue;
  memmove(&aLine[i], &aLine[i + textLen],
	lineLen - i - textLen + 1 /* include trailing NUL */);
  lineLen = strlen(aLine);
  i --; /* restart analysis */
  }
}

#if defined(FLG_PROTOTYPE)
static void processFile(FILE *f, const char fname[])
#else
static void processFile(f, fname)
FILE *f;
const char fname[];
#endif
{
FILE *theText = fopen(fname, "r");
char aLine[STRINGSIZE];
char sectName[STRINGSIZE];

if (theText == NULL) {
  fprintf(stderr, "Warning: unable to read ");
  perror(fname);
  return;
  }
memset(sectName, 0, sizeof(sectName));

for (;;) {
  if (feof(theText) || ferror(theText))
    break;
  if (fgets(aLine, sizeof(aLine), theText) == NULL)
    break;
  removeTextFromLine(aLine, "\\(em");
  removeTextFromLine(aLine, "\\(mi");
  removeTextFromLine(aLine, "\\fI");
  removeTextFromLine(aLine, "\\fR");
  removeBackSpaces(aLine);
  switch (acceptLine(theText, aLine)) {
    case 0:
      continue;

    case 1: { /* .TH; get section name */
      char *p = &aLine[3];
      char *q;

      SKIP_WHITE(p); /* skip white space after .TH */
      SKIP_TEXT(p);  /* skip name of man page */
      SKIP_WHITE(p); /* skip white space after name */
      q = p;
      SKIP_TEXT(p);  /* skip to end of section name */
      *p = '\0';
      strcpy(sectName, q);
      break;
      }

    case 2: { /* .SH NAME; grab descriptive text and make a statement... */
      char *description;
      char *p = &aLine[0];
      char namesect[STRINGSIZE];
      char *name;

      SKIP_WHITE(p);
      name = description = p;
      SKIP_TEXT(p);
      if (*p != '\0') {
	/* null-terminate and use for name */
        *p = '\0';
        p++;

	/* go after rest */
	SKIP_WHITE(p);
        description = p;

        /* but don't print word beginning with a dash */
        if (*p == '-' || *p == '\\' && p[1] == '-') {
	  SKIP_TEXT(p);
          SKIP_WHITE(p);
          description = p;
          }
        }
      /* now blow it out */
      sprintf(namesect, "%s (%s)", name, sectName);
      fprintf(f, "%-23.23s - %s", namesect, description);
			/* note -- fgets has left newline on description */
      goto DONE_FILE;
      }

    default:
      fprintf(stderr, "internal error!\n");
      exit(1);
    } /* switch */
  } /* for */

DONE_FILE:
  fclose(theText);
}

#if defined(FLG_PROTOTYPE)
static void processSubDir(FILE *f, const char dirName[])
#else
static void processSubDir(f, dirName)
FILE *f;
const char dirName[];
#endif
{
  DirectoryEntrySType *dir;
  DIR *dp;

  dp = opendir(dirName);
  if (dp == NULL ) {
    fprintf(stderr, "Warning; directory not readable; ");
    perror(dirName);
    return;
    }

  while ((dir = readdir(dp)) != NULL ) {
    struct stat statbuf;
    char fullName[FILENAMESIZE];

    /* ignore boring entries... */
    if (strcmp(dir->d_name, ".") == 0)
      continue;
    if (strcmp(dir->d_name, "..") == 0)
      continue;

    sprintf(fullName, "%s/%s", dirName, dir->d_name);
    if (stat(fullName, &statbuf) != 0) {
      fprintf(stderr, "Warning: Unable to stat %s\n", fullName);
      continue;
      }
    if (!S_ISREG(statbuf.st_mode)) {
      fprintf(stderr, "Warning: %s is not a regular file\n", fullName);
      continue;
      }
    processFile(f, fullName);
    }
  closedir(dp);
}

#if defined(FLG_PROTOTYPE)
static void processDirectory(const char dirName[])
#else
static void processDirectory(dirName)
const char dirName[];
#endif
{
  DirectoryEntrySType *dir;
  DIR *dp;
  FILE *newWhatis;
  char tempName[FILENAMESIZE];
  char temp2Name[FILENAMESIZE];
  char realName[FILENAMESIZE];

  dp = opendir(dirName);
  if (dp == NULL ) {
    fprintf(stderr, "directory does not exist; ");
    perror(dirName);
    exit(1);
    }
  sprintf(tempName, "%s/makewhatis-%d", dirName, getpid());
  sprintf(temp2Name, "%s/makewhatis2-%d", dirName, getpid());
  sprintf(realName, "%s/whatis", dirName);
  newWhatis = fopen(tempName, "w");
  if (newWhatis == NULL) {
    fprintf(stderr, "cannot open temporary file; ");
    perror(tempName);
    exit(1);
    }

  while ((dir = readdir(dp)) != NULL ) {
    struct stat statbuf;
    char fullName[FILENAMESIZE];

    /* if (dir->d_fileno == 0)        or d_ino */
    if (dir->d_ino == 0)
      continue;

    /* ignore boring entries... */
    if (strcmp(dir->d_name, ".") == 0)
      continue;
    if (strcmp(dir->d_name, "..") == 0)
      continue;
    sprintf(fullName, "%s/%s", dirName, dir->d_name);
    if (stat(fullName, &statbuf) != 0) {
      fprintf(stderr, "Warning: Unable to stat %s\n", fullName);
      continue;
      }
    if (!S_ISDIR(statbuf.st_mode))
      continue;
    processSubDir(newWhatis, fullName);
    }
  closedir(dp);
  fclose(newWhatis);
  {
    char cmd[STRINGSIZE];

    sprintf(cmd, "/bin/sort <%s >%s", tempName, temp2Name);
    if (system(cmd) >> 8 != 0) {
      fprintf(stderr, "error running sort on %s\n", tempName);
      unlink(tempName);
      unlink(temp2Name);
      }
    unlink(tempName);
  }
  if (rename(temp2Name, realName) != 0) {
    fprintf(stderr, "Warning: Unable to rename %s to ", tempName);
    perror(realName);
    unlink(temp2Name);
    }
}

#if defined(FLG_PROTOTYPE)
int main(int argc, char *argv[])
#else
int main(argc, argv)
int argc;
char *argv[];
#endif
{
int i;

if (argc == 1) {
  printf("Usage: %s directory [directory...]\n", argv[0]);
  return 1;
  }

for (i = 1; i < argc; i ++) {
  processDirectory(argv[1]);
  }
return 0;
}
