#define UTILS_C TRUE
/* ========================================================================
 * Name - utils.c
 *
 * Description - 
 *
 * $Id: utils.c,v 1.4 1992/09/16 23:08:09 penneyj Exp $
 *
 * $Log: utils.c,v $
 * Revision 1.4  1992/09/16  23:08:09  penneyj
 * Add version string
 *
 * Revision 1.3  1992/09/16  23:05:07  penneyj
 * better usage message
 *
 * Revision 1.2  1992/09/15  22:37:44  penneyj
 * misc facelift, preparing for distribution on alt.sources
 *
 * Revision 1.1  1992/09/15  22:18:47  penneyj
 * Initial revision
 *
 *========================================================================
 */

#include "general.h"
#include "config.h"
#include "list.h"
#include "main.h"

#include "utils.h" /* this file's interface */

#define VERSION "3.0, $Date: 1992/09/16 23:08:09 $"

#if defined(FLG_PROTOTYPE)
void die(deathEType code)
#else
void die(code)
deathEType code;
#endif
{
if (code == OKEXIT)
  exit(0);
else {
  /* do printf here... */
  exit(1);
  }
}

/*
 *  Sleep the indicated number of microseconds.
 */
#if defined(FLG_PROTOTYPE)
void waitamoment(void)
#else
void waitamoment()
#endif
{
#if 0
  struct timeval tv;

  tv.tv_sec = atime / 1000000;
  tv.tv_usec = atime % 1000000;

  select(0, 0, 0, 0, &tv);
#else
  sleep(1);
#endif
}

/*
 *  Everyone has their own Malloc - me too!
 */
#if defined(FLG_PROTOTYPE)
void *MyMalloc(int space)
#else
void *MyMalloc(space)
int  space;
#endif
{
  char *temp;

  if (space <= 0) {
    warning("MyMalloc used inconsistently; %s", "shutting down");
    kill(getpid(), 11);
    die(INTERNERROR);
    /* NOTREACHED */
    }

  temp = malloc((size_t)space);
  if (temp == NULL) {
    perror("Couldn't malloc!");
    cleanup();
    die(OUTSIDERROR);
    /* NOTREACHED */
    }
  return temp;
}

/*
 *  Gives a name to put tmpfiles in. Puts it in the list of files to
 *  delete.
 */
#if defined(FLG_PROTOTYPE)
const char *tmpfilename(void)
#else
const char *tmpfilename() 
#endif
{
  char *ret;
  int  mypid;
  static int     counter = 0;

  ret = MyMalloc(strlen(TMPDIR) + strlen(MYID) + EXTRASPACE);
  mypid = getpid();
  sprintf(ret, "%s%s%d.%d", TMPDIR, MYID, mypid, counter++);
  ADDTOLIST(tmpfiles, ret);
  return ret;
}


/* 
 * Boolean function matchKey matches basename of a file and a keyword.
 */
#if defined(FLG_PROTOTYPE)
BoolType matchKey(const char *filename, const char *keyword)
#else
BoolType matchKey(filename, keyword)
const char *filename, *keyword;
#endif
{
  while (*filename == *keyword) {
    keyword++;
    filename++;
    }
  return *keyword == '\0' && *filename == '.';
}


/*
 * Prints usage according to mode.
 */
#if defined(FLG_PROTOTYPE)
void prusage(void)
#else
void prusage() 
#endif
{
  fprintf(stderr,
"    Usage:\n\
        %s -adfknruv1 -T macros [section] keyword [keyword...]\n", MAN_NAME);
      fprintf(stderr,
"            -a all pages, even if they seem redundant\n\
            -r force reformatting, even if it doesn't seem necessary\n\
            -t list man pages, but don't print\n\
            -1 only print first match\n\
            -T choose macro package\n");
      fprintf(stderr,
"        %s -dfknuv keyword [keyword...]\n\
        %s -dfknuv keyword [keyword...]\n", WHATIS_NAME, APROPOS_NAME);

/* -i, -X not implemented */
  fprintf(stderr,
"    Arguments for all forms:\n\
            -d debug\n\
            -f args -- force whatis\n\
            -k args -- force apropos\n\
            -n reset -r,-t,-1,-a,-d,-v\n\
            -u this message\n\
            -v verbose\n");

  if (verbos_f)
    printf("\n\nVersion: %s\n", VERSION);
  die(USRERROR);
}

/* 
 *  Searches one directory
 *  with callbacks ad nauseatum.
 */

#if defined(FLG_PROTOTYPE)
int dirsearch(const char *dirname,
	      namecheckfunctype *namecheck,
	      statcheckfunctype *statcheck,
	      founditfunctype *foundit,
	      void *divinfo)
#else
int dirsearch(dirname,namecheck,statcheck,foundit,divinfo)
     char *dirname;
     namecheckfunctype *namecheck;
     statcheckfunctype *statcheck;
     founditfunctype *foundit;
     void *divinfo;
#endif
{
  DIR * dp;
  DirectoryEntrySType *dir;
  char tmpbuf[BUFFERSIZE];
  struct stat statbuf;
  int nametype;
  int total;
  
  total = 0;     
  dp = opendir(dirname);
  if (dp == NULL ) {
    if (verbos_f)
      warning("Cannot open %s", dirname);
    return 0;
    }
  while ((dir = readdir(dp)) != NULL ) {
    /* if (dir->d_fileno == 0)        or d_ino */
    if (dir->d_ino == 0)
      continue;

    nametype = (*namecheck)(dir->d_name, dirname);
    if (nametype == 0)
      continue;

    sprintf(tmpbuf, "%s%s%s", dirname, DIRSEP, dir->d_name);
    if (stat(tmpbuf, &statbuf) != 0) { 
      fprintf(stderr, "Stat failed on: ");
      perror(tmpbuf);
      continue;
      }

    if (!(*statcheck)(&statbuf))
      continue;

    total += (*foundit)(tmpbuf,dir->d_name,nametype,divinfo);
    }
  closedir(dp);
  return total;
}

/* exec a command.  Print status, clean up, etc. */
#if defined(FLG_PROTOTYPE)
BoolType doit(const char todo[], const char *message)
#else
BoolType doit(todo, message)
const char todo[];
const char *message;
#endif
{
  if (message != NULL)
    fprintf(stderr, "%s", message);
  if (debug_f) 
    fprintf(stderr, "Doing `%s`\n", todo);

  childtokill = fork();
  if (childtokill == -1) {
    warning("Fatal error - fork failure, errno = %d", (void *)errno);
    die(FILERROR);
    }
  else
  if (childtokill == 0) { /* child process */
    /* So we don't get two error messages. */
    signal(SIGINT, SIG_DFL);
    if (message != NULL)
      fprintf(stderr, "  Wait... ");
    fflush(stderr);
    putenv((char *)THESHELL);
    system(todo);
    die(OKEXIT);
    }
  wait(NULL);
  notyet = NULL;
  childtokill = 0;
  if (message != NULL)
    fprintf(stderr, "done.\n");
  return TRUE;
}
